"""
Basic tests for MEDUSA
"""
import pytest
from medusa.cli import main


def test_import():
    """Test that we can import the main module"""
    assert main is not None


def test_version():
    """Test version is accessible"""
    from medusa import __version__
    assert __version__ == "0.11.15"
